import sys
import pyslow5 as slow5
import h5py
import time
import statistics
import random


def read_multi_fast5(hdf, filename):
    '''
    read multifast5 file and return data
    '''

    for read in list(hdf.keys()):
        f5_dic = {}
        # read = "read_{}".format(readID)
        f5_dic = {'signal': [], 'readID': '', 'digitisation': 0.0,
                        'offset': 0.0, 'range': 0.0, 'sampling_rate': 0.0}

        f5_dic['signal'] = hdf[read]['Raw/Signal'][()]

        f5_dic['read_id'] = hdf[read]['Raw'].attrs['read_id'].decode()
        f5_dic['digitisation'] = hdf[read]['channel_id'].attrs['digitisation']
        f5_dic['offset'] = hdf[read]['channel_id'].attrs['offset']
        f5_dic['range'] = float("{0:.2f}".format(hdf[read]['channel_id'].attrs['range']))
        f5_dic['sampling_rate'] = hdf[read]['channel_id'].attrs['sampling_rate']

        yield f5_dic


def read_multi_fast5_single(hdf, filename, readID):
    '''
    read multifast5 file and return data
    '''
    f5_dic = {}

    # for read in list(hdf.keys()):
    read = "read_{}".format(readID)
    f5_dic = {'signal': [], 'readID': '', 'digitisation': 0.0,
                    'offset': 0.0, 'range': 0.0, 'sampling_rate': 0.0}

    f5_dic['signal'] = hdf[read]['Raw/Signal'][()]

    f5_dic['read_id'] = hdf[read]['Raw'].attrs['read_id'].decode()
    f5_dic['digitisation'] = hdf[read]['channel_id'].attrs['digitisation']
    f5_dic['offset'] = hdf[read]['channel_id'].attrs['offset']
    f5_dic['range'] = float("{0:.2f}".format(hdf[read]['channel_id'].attrs['range']))
    f5_dic['sampling_rate'] = hdf[read]['channel_id'].attrs['sampling_rate']

    return f5_dic


def read_multi_fast5_list(hdf, filename, readIDs):
    '''
    read multifast5 file and return data
    '''

    for readID in readIDs:
        f5_dic = {}
        read = "read_{}".format(readID)
        f5_dic = {'signal': [], 'readID': '', 'digitisation': 0.0,
                        'offset': 0.0, 'range': 0.0, 'sampling_rate': 0.0}

        f5_dic['signal'] = hdf[read]['Raw/Signal'][()]

        f5_dic['read_id'] = hdf[read]['Raw'].attrs['read_id'].decode()
        f5_dic['digitisation'] = hdf[read]['channel_id'].attrs['digitisation']
        f5_dic['offset'] = hdf[read]['channel_id'].attrs['offset']
        f5_dic['range'] = float("{0:.2f}".format(hdf[read]['channel_id'].attrs['range']))
        f5_dic['sampling_rate'] = hdf[read]['channel_id'].attrs['sampling_rate']

        yield f5_dic


def read_fast5(filename):
    hdf = h5py.File(filename, 'r')
    for read in read_multi_fast5(hdf, filename):
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")
    hdf.close()


def read_slow5(filename):
    s5 = slow5.Open(filename, 'r')
    for read in s5.seq_reads():
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")


def read_fast5_random_single(filename, readIDs):
    hdf = h5py.File(filename, 'r')
    for ID in readIDs:
        read = read_multi_fast5_single(hdf, filename, ID)
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")
    hdf.close()


def read_slow5_random_single(filename, readIDs):
    s5 = slow5.Open(filename, 'r')
    for ID in readIDs:
        read = s5.get_read(ID)
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")


def read_fast5_random_list(filename, readIDs):
    hdf = h5py.File(filename, 'r')
    for read in read_multi_fast5_list(hdf, filename, readIDs):
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")
    hdf.close()


def read_slow5_random_list(filename, readIDs):
    s5 = slow5.Open(filename, 'r')
    for read in s5.get_read_list(readIDs):
        sig = read['signal']
        readID = read['read_id']
        print(f"{readID}: {sig[:10]}")



def get_readIDs(filename):
    IDs = []
    s5 = slow5.Open(filename, 'r')
    for read in s5.seq_reads():
        IDs.append(read['read_id'])
    return IDs



# call stuff here

# files
fast5_file = sys.argv[1]
slow5_file = sys.argv[2]
blow5_file = sys.argv[3]
N = int(sys.argv[4]) # number of rounds for each test

print("|====================================|", file=sys.stderr)
print("|sequential read - 4000 reads, 1 file|", file=sys.stderr)
print("|====================================|", file=sys.stderr)
f5_times = []
s5_times = []
b5_times = []

for i in range(N):
    start_time = time.time()
    read_fast5(fast5_file)
    ttime = round(time.time() - start_time, 4)
    print(f"fast5 test {i}: {ttime} seconds", file=sys.stderr)
    f5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5(slow5_file)
    ttime = round(time.time() - start_time, 4)
    print(f"slow5 test {i}: {ttime} seconds", file=sys.stderr)
    s5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5(blow5_file)
    ttime = round(time.time() - start_time, 4)
    print(f"blow5 test {i}: {ttime} seconds", file=sys.stderr)
    b5_times.append(ttime)


print(f"fast5 mean: {statistics.mean(f5_times)}", file=sys.stderr)
print(f"fast5 stdev: {statistics.stdev(f5_times)}", file=sys.stderr)
print(f"fast5 median: {statistics.median(f5_times)}", file=sys.stderr)
print("================")
print(f"slow5 mean: {statistics.mean(s5_times)}", file=sys.stderr)
print(f"slow5 stdev: {statistics.stdev(s5_times)}", file=sys.stderr)
print(f"slow5 median: {statistics.median(s5_times)}", file=sys.stderr)
print("================")
print(f"blow5 mean: {statistics.mean(b5_times)}", file=sys.stderr)
print(f"blow5 stdev: {statistics.stdev(b5_times)}", file=sys.stderr)
print(f"blow5 median: {statistics.median(b5_times)}", file=sys.stderr)
print("==============================================================")


# ====================================================
print("|=======================================|", file=sys.stderr)
print("|random read single - 4000 reads, 1 file|", file=sys.stderr)
print("|=======================================|", file=sys.stderr)

f5_times = []
s5_times = []
b5_times = []

random.seed(1337)

readIDs = get_readIDs(slow5_file)
random.shuffle(readIDs)


for i in range(N):
    start_time = time.time()
    read_fast5_random_single(fast5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"fast5 test {i}: {ttime} seconds", file=sys.stderr)
    f5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5_random_single(slow5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"slow5 test {i}: {ttime} seconds", file=sys.stderr)
    s5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5_random_single(blow5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"blow5 test {i}: {ttime} seconds", file=sys.stderr)
    b5_times.append(ttime)


print(f"fast5 mean: {statistics.mean(f5_times)}", file=sys.stderr)
print(f"fast5 stdev: {statistics.stdev(f5_times)}", file=sys.stderr)
print(f"fast5 median: {statistics.median(f5_times)}", file=sys.stderr)
print("================")
print(f"slow5 mean: {statistics.mean(s5_times)}", file=sys.stderr)
print(f"slow5 stdev: {statistics.stdev(s5_times)}", file=sys.stderr)
print(f"slow5 median: {statistics.median(s5_times)}", file=sys.stderr)
print("================")
print(f"blow5 mean: {statistics.mean(b5_times)}", file=sys.stderr)
print(f"blow5 stdev: {statistics.stdev(b5_times)}", file=sys.stderr)
print(f"blow5 median: {statistics.median(b5_times)}", file=sys.stderr)
print("==============================================================")

# ====================================================
print("|=====================================|", file=sys.stderr)
print("|random read list - 4000 reads, 1 file|", file=sys.stderr)
print("|=====================================|", file=sys.stderr)

f5_times = []
s5_times = []
b5_times = []

random.seed(1337)

readIDs = get_readIDs(slow5_file)
random.shuffle(readIDs)


for i in range(N):
    start_time = time.time()
    read_fast5_random_list(fast5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"fast5 test {i}: {ttime} seconds", file=sys.stderr)
    f5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5_random_list(slow5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"slow5 test {i}: {ttime} seconds", file=sys.stderr)
    s5_times.append(ttime)

for i in range(N):
    start_time = time.time()
    read_slow5_random_list(blow5_file, readIDs)
    ttime = round(time.time() - start_time, 4)
    print(f"blow5 test {i}: {ttime} seconds", file=sys.stderr)
    b5_times.append(ttime)


print(f"fast5 mean: {statistics.mean(f5_times)}", file=sys.stderr)
print(f"fast5 stdev: {statistics.stdev(f5_times)}", file=sys.stderr)
print(f"fast5 median: {statistics.median(f5_times)}", file=sys.stderr)
print("================")
print(f"slow5 mean: {statistics.mean(s5_times)}", file=sys.stderr)
print(f"slow5 stdev: {statistics.stdev(s5_times)}", file=sys.stderr)
print(f"slow5 median: {statistics.median(s5_times)}", file=sys.stderr)
print("================")
print(f"blow5 mean: {statistics.mean(b5_times)}", file=sys.stderr)
print(f"blow5 stdev: {statistics.stdev(b5_times)}", file=sys.stderr)
print(f"blow5 median: {statistics.median(b5_times)}", file=sys.stderr)
print("==============================================================")



import pyslow5 as slow5
import time
import numpy as np


print("|==============================================|")
print("|             pyslow5 test launch              |")
print("|==============================================|")

#globals
debug = 1


# open file and create object
start_time = time.time()
s5 = slow5.Open('examples/example.slow5','r', DEBUG=debug)
ttime = round(time.time() - start_time, 4)
print(dir(s5))
print("slow5 file opened and object created in: {} seconds".format(ttime))
print("==============================================")

# # Get data for ONE individual read, random access
print("get_read check, r1")

start_time = time.time()
read = s5.get_read("r1", aux=["read_number", "start_mux", "blah"])
ttime = round(time.time() - start_time, 4)
print("get_read in: {} seconds".format(ttime))

# print all fields
print("read_id:", read['read_id'])
print("read_group:", read['read_group'])
print("digitisation:", read['digitisation'])
print("offset:", read['offset'])
print("range:", read['range'])
print("sampling_rate:", read['sampling_rate'])
print("len_raw_signal:", read['len_raw_signal'])
print("signal:", read['signal'][:10])
print("pylen of signal:", len(read['signal']))
print("AUX FIELDS:")
print("read_number:", read["read_number"])
print("start_mux:", read["start_mux"])
print("blah:", read["blah"]) #should be None


print("==============================================")

# Get data for ANOTHER ONE individual read, random access, check memory
print("get_read check, r4, converte to pA")
start_time = time.time()
read = s5.get_read("r4", pA=True)
ttime = round(time.time() - start_time, 4)
print("get_read in: {} seconds".format(ttime))

# print all fields
print("read_id:", read['read_id'])
print("read_group:", read['read_group'])
print("digitisation:", read['digitisation'])
print("offset:", read['offset'])
print("range:", read['range'])
print("sampling_rate:", read['sampling_rate'])
print("len_raw_signal:", read['len_raw_signal'])
print("signal:", read['signal'][:10])
print("pylen of signal:", len(read['signal']))
print("signal value type:", type(read['signal'][0]))
#
print("==============================================")

# get yield object and iterate through reads sequentially
print("seq_reads check, all reads")
start_time = time.time()
print("seq_reads readIDs:")
reads = s5.seq_reads()
print("type check reads:", type(reads))
for read in reads:
    print(read['read_id'])

ttime = round(time.time() - start_time, 4)
print("seq_reads in: {} seconds".format(ttime))

print("==============================================")

# get yield object and iterate through selected reads with random access
print("Yield check, selected reads")
read_list = ["r1", "r3", "null_read", "r5", "r2", "r1"]
start_time = time.time()
selected_reads = s5.get_read_list(read_list)
ttime = round(time.time() - start_time, 4)
print("get_read_list in: {} seconds".format(ttime))
print("yielded readIDs:")
for r, read in zip(read_list,selected_reads):
    if read is not None:
        print(r, read['read_id'])
    else:
        print(r, "read not found, None returned")


print("==============================================")
# get header names
print("Get headder names")
names = s5.get_header_names()
print("header names:")
print(names)
print("==============================================")

# get all headers
print("get_all_headers")
headers = s5.get_all_headers()
print(headers)


print("==============================================")
# Get header attributes
print("Get headder attributes")
attr = "flow_cell_id"
val = s5.get_header_value(attr)
print("flow_cell_id: {}".format(val))
attr = "exp_start_time"
val = s5.get_header_value(attr)
print("exp_start_time: {}".format(val))
attr = "heatsink_temp"
val = s5.get_header_value(attr)
print("heatsink_temp: {}".format(val))
for attr in names:
    val = s5.get_header_value(attr)
    print("{}: {}".format(attr, val))

s5.close()
print("==============================================")
s52 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
print("get_read check, 0d624d4b-671f-40b8-9798-84f2ccc4d7fc")
read1 = s52.get_read("0d624d4b-671f-40b8-9798-84f2ccc4d7fc", aux=["read_number", "start_mux", "blah"])
num_read_groups = s52.get_num_read_groups()
print("number of read groups: {}".format(num_read_groups))
# print all fields
print("read_id:", read1['read_id'])
print("read_group:", read1['read_group'])
print("digitisation:", read1['digitisation'])
print("offset:", read1['offset'])
print("range:", read1['range'])
print("sampling_rate:", read1['sampling_rate'])
print("len_raw_signal:", read1['len_raw_signal'])
print("signal:", read1['signal'][:10])
print("pylen of signal:", len(read1['signal']))
print("AUX FIELDS:")
print("read_number:", read1["read_number"])
print("start_mux:", read1["start_mux"])
print("blah:", read1["blah"]) #should be None

# test for a field that doesn't exist, should return None
read2 = s52.get_read("0d624d4b-671f-40b8-9798-84f2ccc4d7fc", aux="blah")
print("AUX FIELDS:")
print("blah:", read2["blah"]) #should be None

read3 = s52.get_read("0d624d4b-671f-40b8-9798-84f2ccc4d7fc", aux="read_number")
print("AUX FIELDS:")
print("read_number:", read3["read_number"])

print("==============================================")
# get aux names
print("Get aux names")
aux_names = s52.get_aux_names()
print("aux names:")
print(aux_names)

print("==============================================")
# get aux values

print("Get aux types")
aux_types = s52.get_aux_types()
print("aux types:")
print(aux_types)
print("==============================================")

# get all aux fields
read4 = s52.get_read("0d624d4b-671f-40b8-9798-84f2ccc4d7fc", aux="all")
print("AUX FIELDS:")
for name in aux_names:
    print("{}:".format(name), read4[name])

print("==============================================")

print("seq_reads with aux:")
reads = s52.seq_reads(pA=True, aux='all')
print("type check reads:", type(reads))
for read in reads:
    print(read['read_id'])
    print("read_number", read['read_number'])

s52.close()
print("==============================================")

print("write reads no aux")

F = slow5.Open('examples/example_write.slow5','w', DEBUG=debug)
header = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads()

for read in reads:
    record = F.get_empty_record()
    for i in read:
        if i in record:
            record[i] = read[i]
    ret = F.write_record(record)
    print("ret: write_record(): {}".format(ret))

F.close()
s58.close()

print("==============================================")

print("append reads no aux")

F = slow5.Open('examples/example_write_append.blow5','w', rec_press="none", sig_press="none", DEBUG=debug)
header = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads()

read_count = 0
for read in reads:
    record = F.get_empty_record()
    for i in read:
        if i in record:
            record[i] = read[i]
    ret = F.write_record(record)
    print("ret: write_record(): {}".format(ret))

s58.close()
F.close()

F2 = slow5.Open('examples/example_write_append.blow5','a', DEBUG=debug)

print("get_all_headers")
headers = F2.get_all_headers()
print(headers)

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads()

for read in reads:
    record = F2.get_empty_record()
    for i in read:
        if i in record:
            if i == "read_id":
                j = read[i]+"_append"
                record[i] = j
            else:
                record[i] = read[i]
    ret = F2.write_record(record)
    print("ret: write_record(): {}".format(ret))

s58.close()
F2.close()


print("==============================================")
print("write reads with aux")

F = slow5.Open('examples/example_write_aux.blow5','w', DEBUG=debug)
header = F.get_empty_header()
header2 = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

for i in header2:
    header2[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header)
print("ret: write_header(): {}".format(ret))
ret = F.write_header(header2, read_group=1)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

for read in reads:
    record, aux = F.get_empty_record(aux=True)
    for i in read:
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    ret = F.write_record(record, aux)
    print("ret: write_record(): {}".format(ret))

s58.close()
F.close()

print("==============================================")
print("append reads with aux")

F = slow5.Open('examples/example_write_append_aux.blow5','w', rec_press="none", sig_press="none", DEBUG=debug)
header = F.get_empty_header()
header2 = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

for i in header2:
    header2[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header)
print("ret: write_header(): {}".format(ret))
ret = F.write_header(header2, read_group=1)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

for read in reads:
    record, aux = F.get_empty_record(aux=True)
    for i in read:
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    ret = F.write_record(record, aux)
    print("ret: write_record(): {}".format(ret))

s58.close()
F.close()

F2 = slow5.Open('examples/example_write_append_aux.blow5','a', DEBUG=debug)

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

for read in reads:
    record, aux = F2.get_empty_record(aux=True)
    for i in read:
        if i in record:
            if i == "read_id":
                j = read[i]+"_append"
                record[i] = j
            else:
                record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    ret = F2.write_record(record, aux)
    print("ret: write_record(): {}".format(ret))

s58.close()
F2.close()



print("==============================================")

print("seq_reads_multi with aux:")
s53 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s53.seq_reads_multi(threads=2, batchsize=3, pA=True, aux='all')
print("type check reads:", type(reads))
for read in reads:
    print(read['read_id'])
    print("read_number", read['read_number'])

print("==============================================")

print("get_reads_multi with aux:")

read_list = ['r0',
             'r1',
             'r2',
             'r3',
             'r4',
             'r5',
             '0a238451-b9ed-446d-a152-badd074006c4',
             '0d624d4b-671f-40b8-9798-84f2ccc4d7fc']

s53.close()

s53 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s53.get_read_list_multi(read_list, threads=2, batchsize=3, pA=True, aux='all')
print("type check reads:", type(reads))
for read in reads:
    print(read['read_id'])
    print("read_number", read['read_number'])

s53.close()
print("==============================================")
print("write reads with aux multi")

F = slow5.Open('examples/example_write_aux_multi.slow5','w', DEBUG=debug)
header = F.get_empty_header()
header2 = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

for i in header2:
    header2[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header)
print("ret: write_header(): {}".format(ret))
ret = F.write_header(header2, read_group=1)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

records = {}
auxs = {}
for read in reads:
    record, aux = F.get_empty_record(aux=True)
    # record = F.get_empty_record()
    for i in read:
        if i == "read_id":
            readID = read[i]
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    records[readID] = record
    auxs[readID] = aux
print(records)
print(auxs)
ret = F.write_record_batch(records, threads=2, batchsize=3, aux=auxs)
print("ret: write_record(): {}".format(ret))

s58.close()
F.close()

print("==============================================")
print("get all readIDs from index")

s58 = slow5.Open('examples/example2.slow5','r', DEBUG=debug)

read_ids, num_reads = s58.get_read_ids()

print(read_ids)
print("number of reads: {}".format(num_reads))

s58.close()

print("==============================================")
print("get enum fields")

s59 = slow5.Open('examples/adv/example3.blow5','r', DEBUG=debug)

e = s59.get_aux_enum_labels('end_reason')

print(e)

s59.close()

print("==============================================")
print("check can read enum aux field")

s510 = slow5.Open('examples/adv/example3.blow5','r', DEBUG=debug)
reads = s510.seq_reads(aux='all')

e = s510.get_aux_enum_labels('end_reason')

for read in reads:
    print(read['read_id'], read['end_reason'], e[read['end_reason']])

s510.close()

print("==============================================")
print("write reads with aux and end_reason enum")

F = slow5.Open('examples/example_write_aux_enum.blow5','w', DEBUG=debug)
header, end_reason_labels = F.get_empty_header(aux=True)

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header, end_reason_labels=end_reason_labels)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/adv/example3.blow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

for read in reads:
    record, aux = F.get_empty_record(aux=True)
    for i in read:
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    ret = F.write_record(record, aux)
    print("ret: write_record(): {}".format(ret))

s58.close()
F.close()
print("==============================================")
print("write reads with new aux fields")

F = slow5.Open('examples/example_write_new_aux.blow5','w', DEBUG=debug)
header, end_reason_labels = F.get_empty_header(aux=True)

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header, end_reason_labels=end_reason_labels)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/adv/example3.blow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

for read in reads:
    record, aux = F.get_empty_record(aux=True)
    for i in read:
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    aux["tracked_scaling_shift"] = 10.0
    aux["tracked_scaling_scale"] = 20.0
    aux["predicted_scaling_shift"] = 30.0
    aux["predicted_scaling_scale"] = 40.0
    aux["num_reads_since_mux_change"] = 60
    aux["time_since_mux_change"] = 50.0
    aux["num_minknow_events"] = 700
    ret = F.write_record(record, aux)
    print("ret: write_record(): {}".format(ret))

s58.close()
F.close()
print("==============================================")
print("read reads with new aux fields")

F = slow5.Open('examples/example_write_new_aux.blow5','r', DEBUG=debug)

reads = F.seq_reads(aux='all')

for read in reads:
    print("tracked_scaling_shift:", read["tracked_scaling_shift"])
    print("tracked_scaling_scale:", read["tracked_scaling_scale"])
    print("predicted_scaling_shift:", read["predicted_scaling_shift"])
    print("predicted_scaling_scale:", read["predicted_scaling_scale"])
    print("num_reads_since_mux_change:", read["num_reads_since_mux_change"])
    print("time_since_mux_change:", read["time_since_mux_change"])
    print("num_minknow_events:", read["num_minknow_events"])

F.close()

print("==============================================")
print("write reads with aux multi")

F = slow5.Open('examples/example_write_aux_enum_multi.blow5','w', DEBUG=debug)
header, end_reason_labels = F.get_empty_header(aux=True)
header2 = F.get_empty_header()

counter = 0
for i in header:
    header[i] = "test_{}".format(counter)
    counter += 1

for i in header2:
    header2[i] = "test_{}".format(counter)
    counter += 1

ret = F.write_header(header, end_reason_labels=end_reason_labels)
print("ret: write_header(): {}".format(ret))
ret = F.write_header(header2, read_group=1)
print("ret: write_header(): {}".format(ret))

s58 = slow5.Open('examples/adv/example3.blow5','r', DEBUG=debug)
reads = s58.seq_reads(aux='all')

records = {}
auxs = {}
for read in reads:
    record, aux = F.get_empty_record(aux=True)
    # record = F.get_empty_record()
    for i in read:
        if i == "read_id":
            readID = read[i]
        if i in record:
            record[i] = read[i]
        if i in aux:
            aux[i] = read[i]
    records[readID] = record
    auxs[readID] = aux
print(records)
print(auxs)
ret = F.write_record_batch(records, threads=2, batchsize=3, aux=auxs)
print("ret: write_record(): {}".format(ret))

s58.close()
F.close()

print("==============================================")
# print("seq_reads with big file:")
# start_time = time.time()
# s53 = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = s53.seq_reads(pA=True, aux='all')
# # print("type check reads:", type(reads))
# for read in reads:
#     print(read['read_id'])
#     # print("read_number", read['read_number'])
# seq_time = round(time.time() - start_time, 4)
# print("get_read in: {} seconds".format(seq_time))
#
# print("==============================================")
# print("seq_reads_multi with big file:")
# start_time = time.time()
# s53 = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = s53.seq_reads_multi(threads=8, batchsize=10, pA=True, aux='all')
# # print("type check reads:", type(reads))
# for read in reads:
#     print(read['read_id'])
#     # print("read_number", read['read_number'])
# seq_multi_time = round(time.time() - start_time, 4)
# print("get_read in: {} seconds".format(seq_multi_time))
#
# print("==============================================")
#
# print("single seq/write with big file:")
# start_time = time.time()
# sR = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = sR.seq_reads(aux='all')
# sW = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/single_read_write.blow5','w', DEBUG=debug)
#
# header = F.get_empty_header()
#
# counter = 0
# for i in header:
#     header[i] = "test_{}".format(counter)
#     counter += 1
#
# ret = sW.write_header(header)
# print("ret: write_header(): {}".format(ret))
#
# for read in reads:
#     record, aux = sW.get_empty_record(aux=True)
#     for i in read:
#         if i in record:
#             record[i] = read[i]
#         if i in aux:
#             aux[i] = read[i]
#     ret = sW.write_record(record, aux)
#
# single_seq_write_time = round(time.time() - start_time, 4)
# print("single seq/write in: {} seconds".format(single_seq_write_time))
#
# print("==============================================")
# print("multi seq, single write with big file:")
# start_time = time.time()
# sR = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = sR.seq_reads_multi(threads=4, batchsize=100, aux='all')
# sW = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/multi_read_single_write.blow5','w', DEBUG=debug)
# header = F.get_empty_header()
#
# counter = 0
# for i in header:
#     header[i] = "test_{}".format(counter)
#     counter += 1
#
# ret = sW.write_header(header)
# print("ret: write_header(): {}".format(ret))
#
# for read in reads:
#     record, aux = sW.get_empty_record(aux=True)
#     for i in read:
#         if i in record:
#             record[i] = read[i]
#         if i in aux:
#             aux[i] = read[i]
#     ret = sW.write_record(record, aux)
#
# multi_seq_single_write_time = round(time.time() - start_time, 4)
# print("multi seq, single write in: {} seconds".format(multi_seq_single_write_time))
#
# print("==============================================")
# print("single seq, multi write with big file:")
# start_time = time.time()
# sR = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = sR.seq_reads(aux='all')
# sW = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/single_read_multi_write.blow5','w', DEBUG=debug)
#
# header = sW.get_empty_header()
#
# counter = 0
# for i in header:
#     header[i] = "test_{}".format(counter)
#     counter += 1
#
# ret = sW.write_header(header)
# print("ret: write_header(): {}".format(ret))
#
# records = {}
# auxs = {}
# for read in reads:
#     record, aux = sW.get_empty_record(aux=True)
#     for i in read:
#         if i == "read_id":
#             readID = read[i]
#         if i in record:
#             record[i] = read[i]
#         if i in aux:
#             aux[i] = read[i]
#     records[readID] = record
#     auxs[readID] = aux
#     if len(records) >= 100:
#         ret = sW.write_record_batch(records, threads=4, batchsize=100, aux=auxs)
#         records = {}
#         auxs = {}
#
# single_seq_multi_write_time = round(time.time() - start_time, 4)
# print("single seq, multi write in: {} seconds".format(single_seq_multi_write_time))
#
# print("==============================================")
# print("multi seq/write with big file:")
# start_time = time.time()
# sR = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/FAK40634_d1cc054609fe2c5fcdeac358864f9dc81c8bb793_95.blow5','r', DEBUG=debug)
# reads = sR.seq_reads_multi(threads=4, batchsize=100, aux='all')
# sW = slow5.Open('/home/jamfer/Data/SK/multi_fast5/s5/multi_read_write.blow5','w', DEBUG=debug)
#
#
# header = sW.get_empty_header()
#
# counter = 0
# for i in header:
#     header[i] = "test_{}".format(counter)
#     counter += 1
#
# ret = sW.write_header(header)
# print("ret: write_header(): {}".format(ret))
#
# records = {}
# auxs = {}
# for read in reads:
#     record, aux = sW.get_empty_record(aux=True)
#     for i in read:
#         if i == "read_id":
#             readID = read[i]
#         if i in record:
#             record[i] = read[i]
#         if i in aux:
#             aux[i] = read[i]
#     records[readID] = record
#     auxs[readID] = aux
#     if len(records) >= 100:
#         ret = sW.write_record_batch(records, threads=4, batchsize=100, aux=auxs)
#         records = {}
#         auxs = {}
# multi_seq_write_time = round(time.time() - start_time, 4)
# print("multi seq/write in: {} seconds".format(multi_seq_write_time))
#
# print("==============================================")
#
# print("big file times:")
# print("single seq/write in: {} seconds".format(single_seq_write_time))
# print("multi seq, single write in: {} seconds".format(multi_seq_single_write_time))
# print("single seq, multi write in: {} seconds".format(single_seq_multi_write_time))
# print("multi seq/write in: {} seconds".format(multi_seq_write_time))

print("done")



try:
    from setuptools import setup, Extension
    from setuptools.command.install import install

except ImportError:
    from distutils.core import setup
    from distutils.extension import Extension

import sys
import platform
import os

include_dirs = []
# import numpy as np

# creat dummy closures for metadata reading on first parse of setup.py
# that way it picks up the requirements and installs them, then can use them
# for the install.
try:
    import numpy as np
    include_dirs = ['include/', np.get_include(), 'thirdparty/streamvbyte/include']
except ImportError:
    include_dirs = ['include/', 'thirdparty/streamvbyte/include']
    def np(*args, ** kwargs ):
        import numpy as np
        return np(*args, ** kwargs)

try:
    from Cython.Build import build_ext
except ImportError:
    def build_ext(*args, ** kwargs ):
        from Cython.Build import build_ext
        return build_ext(*args, ** kwargs)

# from Cython.Build import build_ext

#adapted from https://github.com/lh3/minimap2/blob/master/setup.py

sources=['python/pyslow5.pyx', 'src/slow5.c', 'src/slow5_press.c', 'src/slow5_misc.c', 'src/slow5_idx.c',
            'src/slow5_mt.c',
            'thirdparty/streamvbyte/src/streamvbyte_zigzag.c', 'thirdparty/streamvbyte/src/streamvbyte_decode.c', 'thirdparty/streamvbyte/src/streamvbyte_encode.c']
depends=['python/pyslow5.pxd', 'python/pyslow5.h',
            'slow5/slow5.h', 'slow5/slow5_defs.h', 'slow5/slow5_error.h', 'slow5/slow5_press.h', 'slow5/slow5_lazymt.h',
            'slow5/klib/khash.h', 'slow5/klib/kvec.h',
            'src/slow5_extra.h', 'src/slow5_idx.h', 'src/slow5_misc.h', 'src/klib/ksort.h',
            'thirdparty/streamvbyte/include/streamvbyte.h', 'thirdparty/streamvbyte/include/streamvbyte_zigzag.h']
extra_compile_args = ['-g', '-Wall', '-O2', '-std=c99', '-DSLOW5_ENABLE_MT=1' ]
# extra_compile_args = []
# os.environ["CFLAGS"] = '-g -Wall -O2 -std=c99'

if platform.system() != 'Darwin':
    arch=platform.machine()
    if arch in ["aarch64"]:
        extra_compile_args.append('-D__ARM_NEON__')
    elif arch in ["armv7l"]:
        extra_compile_args.append('-mfpu=neon')
    elif arch in ["x86_64"]:
        extra_compile_args.extend(['-DSTREAMVBYTE_SSSE3=1', '-mssse3'])   # WARNING: ancient x86_64 CPUs don't have SSSE3
# note that on macos, for now we just let it compile without simd

# include_dirs = ['include/', np.get_include(), 'thirdparty/streamvbyte/include']
libraries = ['m', 'z']
library_dirs = ['.']

# a nasty hack to provide option to build with zstd
zstd=0
try:
    zstd=os.environ["PYSLOW5_ZSTD"]
except:
    zstd=0

if zstd=="1":
    extra_compile_args.append('-DSLOW5_USE_ZSTD=1')
    libraries.append('zstd')

extensions = [Extension('pyslow5',
                  sources = sources,
                  depends = depends,
                  extra_compile_args = extra_compile_args,
                  libraries = libraries,
                  include_dirs = include_dirs,
                  library_dirs = library_dirs,
                  language = 'c' )]

def readme():
	with open('docs/pyslow5_api/pyslow5.md') as f:
		return f.read()


setup(
    name = 'pyslow5',
    version='1.3.0',
    url = 'https://github.com/hasindu2008/slow5lib',
    description='slow5lib python bindings',
    long_description=readme(),
    long_description_content_type='text/markdown',
    author='Hasindu Gamaarachchi, Sasha Jenner, James Ferguson',
    author_email='hasindu2008@gmail.com',
    maintainer='Hasindu Gamaarachchi',
    maintainer_email='hasindu2008@gmail.com',
    license = 'MIT',
    keywords = ['nanopore','slow5','signal'],
    ext_modules=extensions,
    cmdclass= {'build_ext': build_ext},
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: C',
        'Programming Language :: Cython',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics'],
    python_requires='>=3.4.3',
    install_requires=["numpy"],
    setup_requires=["Cython", "numpy"]
    )
